Imports System.Xml

Friend Class ClsAlbum
    Inherits ClsBaseXml
    Implements IPath

    Private mAlbums As New SortedList(Of String, ClsAlbum)
    Private mFichiers As New SortedList(Of String, ClsFichier)
    Private mName As String
    Private mPath As String
    Private mSelected As Boolean

    Private mNode As TreeNode = Nothing
    Private mAudio As ClsAudio = Nothing

    Private mInPropertySet As Boolean = False

#Region "Construction/destruction"

    Friend Sub New(ByVal pParent As ClsBaseXml, ByVal pNode As XmlNode)
        MyBase.New(pParent, pNode)

        Application.DoEvents()

        mName = MyBase.GetAttribut("name")
        mPath = IO.Path.Combine(DirectCast(Parent, IPath).Path, mName)

        If IO.Directory.Exists(mPath) = False Then
            MyBase.DelNode()
            Throw New Exception("Le dossier """ + mPath + """ n'existe pas")
        End If

        Try
            mSelected = CBool(MyBase.GetAttribut("selected"))
        Catch ex As Exception
            mSelected = False
        End Try
        EvtSelChange()

        Dim lBoo1 As Boolean = True
        Do While lBoo1
            lBoo1 = False
            For Each lNode As XmlNode In Node.ChildNodes
                Try
                    If lNode.Name = "album" Then
                        Dim lAlb As ClsAlbum = New ClsAlbum(Me, lNode)
                        mAlbums.Add(lAlb.Name, lAlb)
                    ElseIf lNode.Name = "file" Then
                        Dim lFil As ClsFichier = New ClsFichier(Me, lNode)
                        mFichiers.Add(lFil.ToString, lFil)
                    End If
                Catch ex As Exception
                    If Not (TypeOf ex Is ArgumentException) Then lBoo1 = True
                End Try
            Next
            If lBoo1 Then
                If mAlbums IsNot Nothing Then
                    For Each lAlbum As ClsAlbum In mAlbums.Values
                        lAlbum.Dispose()
                    Next
                    mAlbums.Clear()
                End If
                If mFichiers IsNot Nothing Then
                    For Each lfichier As ClsFichier In mFichiers.Values
                        lfichier.Dispose()
                    Next
                    mFichiers.Clear()
                End If
            End If
        Loop
        'Ajout des rpertoires non rfrencs
        Dim lDir As New IO.DirectoryInfo(Path)
        If lDir.GetDirectories.Length <> mAlbums.Count Then
            For Each llDir As IO.DirectoryInfo In lDir.GetDirectories
                Dim lBoo As Boolean = False
                For Each lAlbum As ClsAlbum In mAlbums.Values
                    If lAlbum.Path.ToLower = llDir.FullName.ToLower Then
                        lBoo = True
                        Exit For
                    End If
                Next
                If Not lBoo Then
                    Dim lAlb As ClsAlbum = New ClsAlbum(Me, llDir)
                    mAlbums.Add(lAlb.Name, lAlb)
                End If
            Next
        End If
        Application.DoEvents()
        'Ajout des fichiers non rfrencs
        Dim ltFiles As New List(Of IO.FileInfo)
        Dim ltExt As String() = "wav;mp3;wma;mid".Split(";"c)
        For Each lExt As String In ltExt
            ltFiles.AddRange(lDir.GetFiles("*." + lExt))
        Next
        If ltFiles.Count <> mFichiers.Count Then
            For Each lFile As IO.FileInfo In ltFiles
                Dim lBoo As Boolean = False
                For Each lFichier As ClsFichier In mFichiers.Values
                    If lFile.FullName.ToLower = lFichier.FileName.ToLower Then
                        lBoo = True
                        Exit For
                    End If
                Next
                If Not lBoo Then
                    Dim lFil As ClsFichier = New ClsFichier(Me, lFile)
                    mFichiers.Add(lFil.ToString, lFil)
                End If
            Next
        End If
    End Sub

    Friend Sub New(ByVal pParent As ClsBaseXml, ByVal pDir As IO.DirectoryInfo)
        MyBase.New(pParent, pParent.Document.CreateElement("album"))
        If pParent.Node Is Nothing Then
            pParent.Document.DocumentElement.AppendChild(Node)
        Else
            pParent.Node.AppendChild(Node)
        End If

        mName = pDir.Name
        SetAttribut("name", mName)
        mPath = pDir.FullName

        SetAttribut("selected", "0")
        mSelected = False

        'Ajout des rpertoires non rfrencs
        If pDir.GetDirectories.Length <> mAlbums.Count Then
            For Each llDir As IO.DirectoryInfo In pDir.GetDirectories
                Dim lBoo As Boolean = False
                For Each lAlbum As ClsAlbum In mAlbums.Values
                    If lAlbum.Path.ToLower = llDir.FullName.ToLower Then
                        lBoo = True
                        Exit For
                    End If
                Next
                If Not lBoo Then
                    Dim lAlb As ClsAlbum = New ClsAlbum(Me, llDir)
                    mAlbums.Add(lAlb.Name, lAlb)
                End If
            Next
        End If
        'Ajout des fichiers non rfrencs
        Dim ltFiles As New List(Of IO.FileInfo)
        Dim ltExt As String() = "wav;mp3;wma;mid".Split(";"c)
        For Each lExt As String In ltExt
            ltFiles.AddRange(pDir.GetFiles("*." + lExt))
        Next
        Application.DoEvents()
        If ltFiles.Count <> mFichiers.Count Then
            For Each lFile As IO.FileInfo In ltFiles
                Dim lBoo As Boolean = False
                For Each lFichier As ClsFichier In mFichiers.Values
                    If lFile.FullName.ToLower = lFichier.FileName.ToLower Then
                        lBoo = True
                        Exit For
                    End If
                Next
                If Not lBoo Then
                    Dim lFil As ClsFichier = New ClsFichier(Me, lFile)
                    Try
                        mFichiers.Add(lFil.ToString, lFil)
                    Catch ex As Exception
                        AddLog("ClsAlbum.New", "Exception non gre : " + ex.ToString, EPrio.Avertissement)
                    End Try
                End If
            Next
        End If
    End Sub

    Friend Overrides Sub Dispose()
        If mAlbums IsNot Nothing Then
            For Each lAlbum As ClsAlbum In mAlbums.Values
                lAlbum.Dispose()
            Next
            mAlbums.Clear()
        End If
        mAlbums = Nothing
        If mFichiers IsNot Nothing Then
            For Each lfichier As ClsFichier In mFichiers.Values
                lfichier.Dispose()
            Next
            mFichiers.Clear()
        End If
        mFichiers = Nothing
        MyBase.Dispose()
    End Sub
#End Region

#Region "Membres publiques"

    Friend Function GetTreeNode() As TreeNode
        ThrowDispose()
        Dim lList As New List(Of TreeNode)
        Dim lSel As Boolean = True
        For Each lObj As ClsAlbum In mAlbums.Values
            Dim lTreeNode As TreeNode = lObj.GetTreeNode
            If lTreeNode IsNot Nothing Then lList.Add(lTreeNode)
            If lObj.Selected = False Then lSel = False
        Next
        Dim lNode As TreeNode
        If lList.Count = 0 Then
            If mFichiers.Count = 0 Then
                Return Nothing
            Else
                lNode = New TreeNode(mName, 1, 1)
            End If
        Else
            lNode = New TreeNode(mName, 1, 1, lList.ToArray)
        End If
        If mSelected <> lSel And lList.Count > 0 Then SetSel(lSel)
        lNode.Checked = mSelected
        lNode.Tag = Me
        mNode = lNode
        Return lNode
    End Function

    Public ReadOnly Property Path() As String Implements IPath.Path
        Get
            Return mPath
        End Get
    End Property

    Public Sub Save(Optional ByVal pShowMessage As Boolean = True) Implements IPath.Save
        ThrowDispose()
        CType(Parent, IPath).Save(pShowMessage)
    End Sub

    Public ReadOnly Property Albums() As SortedList(Of String, ClsAlbum)
        Get
            Return mAlbums
        End Get
    End Property

    Public ReadOnly Property Fichiers() As SortedList(Of String, ClsFichier)
        Get
            Return mFichiers
        End Get
    End Property

    Public ReadOnly Property Name() As String
        Get
            Return mName
        End Get
    End Property

    Public Property Selected() As Boolean
        Get
            Return mSelected
        End Get
        Set(ByVal value As Boolean)
            If mSelected = value Then Exit Property
            If mInPropertySet Then Exit Property
            mInPropertySet = True
            mSelected = value
            For Each lObj As ClsAlbum In mAlbums.Values
                lObj.Selected = value
            Next
            If mNode IsNot Nothing Then mNode.Checked = value
            If TypeOf Parent Is ClsAlbum Then
                DirectCast(Parent, ClsAlbum).CtrlSel()
            End If
            mSelected = value
            If mSelected Then
                SetAttribut("selected", "1")
            Else
                SetAttribut("selected", "0")
            End If
            mInPropertySet = False
            EvtSelChange()
        End Set
    End Property

    Private Sub SetSel(ByVal pValue As Boolean)
        If mSelected = pValue Then Exit Sub
        mSelected = pValue
        If mInPropertySet = False Then
            If TypeOf Parent Is ClsAlbum Then
                DirectCast(Parent, ClsAlbum).CtrlSel()
            End If
        End If
        If mNode IsNot Nothing Then mNode.Checked = mSelected
        If mSelected Then
            SetAttribut("selected", "1")
        Else
            SetAttribut("selected", "0")
        End If
        EvtSelChange()
    End Sub

    Private Sub CtrlSel()
        Dim lSel As Boolean = True
        For Each lObj As ClsAlbum In mAlbums.Values
            If lObj.Selected = False Then lSel = False
        Next
        If mSelected <> lSel And mAlbums.Count > 0 Then SetSel(lSel)
        If mSelected Then
            SetAttribut("selected", "1")
        Else
            SetAttribut("selected", "0")
        End If
    End Sub

    Private Sub EvtSelChange()
        If mAudio IsNot Nothing Then
            If mSelected Then
                mAudio.ToRead.AddRange(mFichiers.Values)
            Else
                If mFichiers.Values.IndexOf(mAudio.Reading) > -1 Then
                    mAudio.MusicStop()
                End If
                SyncLock mAudio.ToRead
                    For Each lFic As ClsFichier In mFichiers.Values
                        mAudio.ToRead.Remove(lFic)
                    Next
                End SyncLock
                SyncLock mAudio.Readed
                    For Each lFic As ClsFichier In mFichiers.Values
                        mAudio.Readed.Remove(lFic)
                    Next
                End SyncLock
            End If
        End If
    End Sub

    Public Property Audio() As ClsAudio
        Get
            Return mAudio
        End Get
        Set(ByVal value As ClsAudio)
            mAudio = value
            For Each lAlbum As ClsAlbum In mAlbums.Values
                lAlbum.Audio = value
            Next
            If mAudio IsNot Nothing AndAlso mSelected Then
                mAudio.ToRead.AddRange(mFichiers.Values)
            End If
        End Set
    End Property
#End Region
End Class
