Imports System.Xml

Friend Class ClsDocument
    Inherits ClsBaseXml
    Implements IPath

    Private mAudio As ClsAudio = Nothing
    Private mFileName As String

    Private mAlbums As New SortedList(Of String, ClsAlbum)
#Region "Construction / destruction"
    ''' <summary>
    ''' Ouvre ou cr un fichier XML dont le nom est pass en paramtre
    ''' </summary>
    ''' <param name="pFilename">Nom du fichier  ouvrir ou  crer</param>
    ''' <remarks>Si le fichier n'existe pas ou est endomag, il est cr sinon il est ouvert</remarks>
    Public Sub New(ByVal pFilename As String)
        MyBase.New(New XmlDocument())
        mFileName = pFilename

        If IO.File.Exists(mFileName) Then
            Document.Load(mFileName)
        Else
            Document.LoadXml(My.Resources.NewXml)
        End If
        LoadData()
    End Sub

    Private Sub LoadData()
        If Document.DocumentElement.Name <> "playlist" Then
            If MessageBox.Show("Le fichier de playlist est endomag ou incorrecte, la balise d'entre est diffrente de celle attendue, voulez-vous rinitailiser le fichier ?", "Erreur de chargement", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) = DialogResult.Yes Then
                Document.LoadXml(My.Resources.NewXml)
            Else
                Dispose()
                Throw New Exception("Exception pour quitter le programme, le fichier  charger est incorrecte")
                Application.Exit()
            End If
            Exit Sub
        End If
        Dim lBoo1 As Boolean = True
        Do While lBoo1
            lBoo1 = False
            Application.DoEvents()
            For Each lNode As XmlNode In Document.DocumentElement.ChildNodes
                If lNode.Name = "album" Then
                    Try
                        Dim lAlb As ClsAlbum = New ClsAlbum(Me, lNode)
                        mAlbums.Add(lAlb.Name, lAlb)
                        Application.DoEvents()
                    Catch ex As Exception
                        lBoo1 = True
                    End Try
                End If
            Next
            If lBoo1 Then
                If mAlbums IsNot Nothing Then
                    For Each lAlbum As ClsAlbum In mAlbums.Values
                        lAlbum.Dispose()
                    Next
                    mAlbums.Clear()
                End If
            End If
        Loop
        Dim lDir As IO.DirectoryInfo = (New IO.FileInfo(mFileName)).Directory
        If lDir.GetDirectories.Length <> mAlbums.Count Then
            For Each llDir As IO.DirectoryInfo In lDir.GetDirectories
                Dim lBoo As Boolean = False
                For Each lAlbum As ClsAlbum In mAlbums.Values
                    If lAlbum.Path.ToLower = llDir.FullName.ToLower Then
                        lBoo = True
                        Exit For
                    End If
                Next
                If Not lBoo Then
                    Dim lAlb As ClsAlbum = New ClsAlbum(Me, llDir)
                    mAlbums.Add(lAlb.Name, lAlb)
                End If
            Next
        End If

        System.GC.Collect()
    End Sub

    Protected Overrides Sub Finalize()
        Save(False)
        MyBase.Finalize()
    End Sub
    ''' <summary>
    ''' Cette mthode sert  librer les ressources, elle doit tre appele pour une destruction correcte de la classe par le GC.
    ''' </summary>
    ''' <remarks></remarks>
    Friend Overrides Sub Dispose()
        If mAlbums IsNot Nothing Then
            For Each lAlbum As ClsAlbum In mAlbums.Values
                lAlbum.Dispose()
            Next
            mAlbums.Clear()
        End If
        mAlbums = Nothing
        Save(True)
        MyBase.Dispose()
    End Sub
#End Region

#Region "membres publiques"

    Public Function GetTreeNode() As System.Windows.Forms.TreeNode
        ThrowDispose()
        Dim lList As New List(Of TreeNode)
        For Each lObj As ClsAlbum In mAlbums.Values
            Dim lTreeNode As TreeNode = lObj.GetTreeNode
            If lTreeNode IsNot Nothing Then lList.Add(lTreeNode)
        Next
        Dim lNode As TreeNode
        If lList.Count = 0 Then
            lNode = New TreeNode("Racine", 0, 0, New TreeNode() {New TreeNode("Rien  afficher")})
        Else
            lNode = New TreeNode("Racine", 0, 0, lList.ToArray)
        End If
        lNode.Tag = Me
        Return lNode
    End Function

    Public ReadOnly Property Path() As String Implements IPath.Path
        Get
            ThrowDispose()
            Return (New IO.FileInfo(mFileName)).DirectoryName
        End Get
    End Property

    Public ReadOnly Property Fichiers() As SortedList(Of String, ClsAlbum)
        Get
            ThrowDispose()
            Return mAlbums
        End Get
    End Property

    Public ReadOnly Property FileName() As String
        Get
            ThrowDispose()
            Return mFileName
        End Get
    End Property

    Public Sub Save(Optional ByVal pShowMessage As Boolean = True) Implements IPath.Save
        Try
            If Document IsNot Nothing Then MyBase.Document.Save(mFileName)
        Catch ex As Exception
            AddLog("ClsDocument.Save", "Exception non gre : " + ex.ToString, EPrio.Avertissement)
            If pShowMessage Then MessageBox.Show("Une erreur est survenue lors de l'enregistrement de la playlist : " + ex.Message, "Erreur enregistrement", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    Public Property Audio() As ClsAudio
        Get
            Return mAudio
        End Get
        Set(ByVal value As ClsAudio)
            mAudio = value
            For Each lAlbum As ClsAlbum In mAlbums.Values
                lAlbum.Audio = value
            Next
        End Set
    End Property
#End Region
End Class
