Imports System.Xml

Friend Class ClsFichier
    Inherits ClsBaseXml

    Private mFileName As String
    Private mTextFile As String
    Private mReaded As Boolean

    Private mAuteur As String = ""
    Private mAlbum As String = ""
    Private mTitle As String = ""

#Region "Construction / destruction"

    Friend Sub New(ByVal pParent As ClsAlbum, ByVal pNode As XmlNode)
        MyBase.New(pParent, pNode)

        mFileName = IO.Path.Combine(pParent.Path, GetAttribut("url"))
        If IO.File.Exists(mFileName) = False Then
            MyBase.DelNode()
            Throw New Exception("Le fichier """ + mFileName + """ n'existe pas")
        End If

        mTextFile = GetAttribut("texte")
        If mTextFile = "" Then
            mTextFile = System.IO.Path.GetFileNameWithoutExtension(mFileName) + ".txt"
            mTextFile = IO.Path.Combine(pParent.Path, mTextFile)
        Else
            mTextFile = IO.Path.Combine(pParent.Path, mTextFile)
        End If

        Try
            mReaded = CBool(GetAttribut("read"))
        Catch ex As Exception
            mReaded = False
        End Try

        Dim lStr() As String = IO.Path.GetFileNameWithoutExtension(mFileName).Split(New String() {" - "}, StringSplitOptions.RemoveEmptyEntries)

        Select Case lStr.Length
            Case 1
                mAuteur = ""
                mAlbum = ""
                mTitle = lStr(0)
            Case 2
                mAuteur = lStr(0)
                mAlbum = ""
                mTitle = lStr(1)
            Case 3
                mAuteur = lStr(0)
                mAlbum = lStr(1)
                mTitle = lStr(2)
            Case Else
                mAuteur = "Non rcuprable"
                mAlbum = "Non rcuprable"
                mTitle = "Non rcuprable"
        End Select
    End Sub

    Friend Sub New(ByVal pParent As ClsAlbum, ByVal pFichier As IO.FileInfo)
        MyBase.New(pParent, pParent.Document.CreateElement("file"))
        'Undone, ajout du nouvel lment

        pParent.Node.AppendChild(Node)

        mFileName = pFichier.FullName

        SetAttribut("url", pFichier.Name)

        SetAttribut("texte", pFichier.Name)
        If mTextFile = "" Then
            mTextFile = System.IO.Path.GetFileNameWithoutExtension(mFileName) + ".txt"
            mTextFile = IO.Path.Combine(pParent.Path, mTextFile)
        Else
            mTextFile = IO.Path.Combine(pParent.Path, mTextFile)
        End If


        mReaded = False
        SetAttribut("read", "0")

        Dim lStr() As String = IO.Path.GetFileNameWithoutExtension(mFileName).Split(New String() {" - "}, StringSplitOptions.RemoveEmptyEntries)

        Select Case lStr.Length
            Case 1
                mAuteur = ""
                mAlbum = ""
                mTitle = lStr(0)
            Case 2
                mAuteur = lStr(0)
                mAlbum = ""
                mTitle = lStr(1)
            Case 3
                mAuteur = lStr(0)
                mAlbum = lStr(1)
                mTitle = lStr(2)
            Case Else
                mAuteur = "Non rcuprable"
                mAlbum = "Non rcuprable"
                mTitle = "Non rcuprable"
        End Select
    End Sub

    Friend Overrides Sub Dispose()
        If IO.File.Exists(mTextFile) = False Then
            SetAttribut("texte", "")
        Else
            SetAttribut("texte", IO.Path.GetFileName(mTextFile))
        End If
        mFileName = ""
        mTextFile = ""
        mAuteur = ""
        mAlbum = ""
        mTitle = ""
        MyBase.Dispose()
    End Sub
#End Region

#Region "Membres publiques"

    Public ReadOnly Property FileName() As String
        Get
            ThrowDispose()
            Return mFileName
        End Get
    End Property

    Public ReadOnly Property FileTitle() As String
        Get
            ThrowDispose()
            Return IO.Path.GetFileNameWithoutExtension(mFileName)
        End Get
    End Property

    ''' <summary>
    ''' Retourne ou dfinit le texte de la piste
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Texte() As String
        Get
            ThrowDispose()
            Try
                Return FileToTxt(IO.File.ReadAllText(mTextFile, System.Text.Encoding.UTF8))
            Catch ex As Exception
                Return ""
            End Try
        End Get
        Set(ByVal value As String)
            ThrowDispose()
            Try
                If value = "" Then
                    IO.File.Delete(mTextFile)
                Else
                    IO.File.WriteAllText(mTextFile, TxtToFile(value), System.Text.Encoding.UTF8)
                End If
            Catch ex As Exception

            End Try
        End Set
    End Property

    Public Property Readed() As Boolean
        Get
            ThrowDispose()
            Return mReaded
        End Get
        Set(ByVal value As Boolean)
            ThrowDispose()
            mReaded = value
            If value Then
                SetAttribut("read", "1")
            Else
                SetAttribut("read", "0")
            End If
        End Set
    End Property

    Public ReadOnly Property Auteur() As String
        Get
            Return mAuteur
        End Get
    End Property

    Public ReadOnly Property Album() As String
        Get
            Return mAlbum
        End Get
    End Property

    Public ReadOnly Property Title() As String
        Get
            Return mTitle
        End Get
    End Property

    Public ReadOnly Property Exist() As Boolean
        Get
            Return IO.File.Exists(mFileName)
        End Get
    End Property

    Public Overrides Function ToString() As String
        Return IO.Path.GetFileNameWithoutExtension(mFileName)
    End Function
#End Region
#Region "membres privs"
    ''' <summary>
    ''' Traduis le texte pour l'enregistr dans un fichier
    ''' </summary>
    ''' <param name="pSrc">Texte</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function TxtToFile(ByVal pSrc As String) As String
        'Dim lForgotChars As String = "<>&%"
        'Dim lDst As New System.Text.StringBuilder

        'Dim lCar As String

        'For i As Integer = 0 To pSrc.Length - 1
        'lCar = pSrc.Substring(i, 1)
        'If lForgotChars.Contains(lCar) OrElse AscW(lCar) > 126 Then
        'lCar = "%" + Right("0000" + Hex(AscW(lCar)), 4)
        'End If
        'lDst.Append(lCar)
        'Next

        'Return lDst.ToString
        Return pSrc
    End Function
    ''' <summary>
    ''' Traduis le texte du fichier pour l'affichage
    ''' </summary>
    ''' <param name="pSrc">Contenu du fihier</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function FileToTxt(ByVal pSrc As String) As String
        Dim lDst As New System.Text.StringBuilder

        Dim lCar As String

        For i As Integer = 0 To pSrc.Length - 1
            lCar = pSrc.Substring(i, 1)
            If pSrc.Length - i > 5 AndAlso pSrc.Substring(i, 5).ToLower Like "%[0-9a-f][0-9a-f][0-9a-f][0-9a-f]" Then
                lCar = ChrW(CInt(Val("&h" + pSrc.Substring(i + 1, 4))))
                i += 4
            End If
            lDst.Append(lCar)
        Next

        Return lDst.ToString
    End Function
#End Region
End Class
